from flask_login import login_user, logout_user  # type: ignore
from flask import Blueprint, url_for, redirect, session, current_app
from datetime import timedelta
from compendium_v2.auth import get_client
from compendium_v2.auth.session_management import fetch_user, create_user

routes = Blueprint('authentication', __name__)


@routes.before_request
def before_request():
    # logout users if they haven't visited in 7 days (session refreshes on each visit)
    session.permanent = True
    current_app.permanent_session_lifetime = timedelta(days=7)


@routes.route('/login')
def login():
    client = get_client()

    # _external uses headers to determine the full URL when behind a reverse proxy
    redirect_uri = url_for('authentication.authorize', _external=True)
    return client.authorize_redirect(redirect_uri)


@routes.route('/authorize')
def authorize():
    client = get_client()
    token = client.authorize_access_token()

    profile = client.userinfo(token=token)

    if 'email' not in profile or not profile['email']:
        return '<h3>Authentication failed: Invalid user response from provider</h3>', 400

    user = fetch_user(profile)
    if user is None:
        # create a new user
        user = create_user(profile['email'], profile['name'], profile['sub'])
    login_user(user)

    # redirect to /
    return redirect(url_for('compendium-v2-default.survey_index'))


@routes.route("/logout")
def logout():
    # The user will be logged out of the application, but not the IDP.
    # If they visit again before their oauth token expires, they are immediately logged in.
    logout_user()
    return redirect(url_for('compendium-v2-default.survey_index'))
