import pkg_resources
from flask import Blueprint, jsonify, render_template, Response
from compendium_v2.routes import common

routes = Blueprint('compendium-v2-default', __name__)
API_VERSION = '0.1'

VERSION_SCHEMA = {
    '$schema': 'http://json-schema.org/draft-07/schema#',

    'type': 'object',
    'properties': {
        'api': {
            'type': 'string',
            'pattern': r'\d+\.\d+'
        },
        'module': {
            'type': 'string',
            'pattern': r'\d+\.\d+'
        }
    },
    'required': ['api', 'module'],
    'additionalProperties': False
}


@routes.after_request
def after_request(resp):
    return common.after_request(resp)


@routes.route('/', defaults={'path': ''}, methods=['GET'])
@routes.route('/<path:path>', methods=['GET'])
def index(path):
    is_api = path.startswith('api')

    if is_api:
        # return 404 for API requests that don't match a route
        return Response(status=404, mimetype='application/json', response='{"error": "Not Found"}')

    # fallback to serving the SPA through index.html for all other requests
    # https://flask.palletsprojects.com/en/2.0.x/patterns/singlepageapplications/
    return render_template('index.html')


@routes.route('/survey', defaults={'path': ''}, methods=['GET'])
@routes.route('/survey/', defaults={'path': ''}, methods=['GET'])
@routes.route('/survey/<path:path>', methods=['GET'])
def survey_index(path):
    is_api = path.startswith('api')

    if is_api:
        # return 404 for API requests that don't match a route
        return Response(status=404, mimetype='application/json', response='{"error": "Not Found"}')

    # fallback to serving the SPA through index.html for all other requests
    # https://flask.palletsprojects.com/en/2.0.x/patterns/singlepageapplications/
    return render_template('survey-index.html')


@routes.route('/version', methods=['GET', 'POST'])
@common.require_accepts_json
def version():
    """
    handler for /version requests

    response will be formatted as:

    .. asjson::
        compendium_v2.routes.default.VERSION_SCHEMA

    :return:
    """
    version_params = {
        'api': API_VERSION,
        'module':
            pkg_resources.get_distribution('compendium-v2').version
    }
    return jsonify(version_params)
