import React, { ReactElement } from "react";
import { BrowserRouter as Router, Routes, Route } from "react-router-dom";

import SurveyManagementComponent from './SurveyManagementComponent';
import UserManagementComponent from './UserManagementComponent';
import SurveyContainerComponent from "./SurveyContainerComponent";
import ExternalPageNavBar from "shared/ExternalPageNavBar"
import UserProvider from "./providers/UserProvider";
import Landing from "./Landing";

function App(): ReactElement {
    return (
        <div className="app">
            <Router>
                <UserProvider>
                    <ExternalPageNavBar />
                    <Routes>
                        <Route path="survey/admin/surveys" element={<SurveyManagementComponent />} />
                        <Route path="survey/admin/users" element={<UserManagementComponent />} />
                        <Route path="survey/admin/inspect/:year" element={<SurveyContainerComponent loadFrom={'/api/survey/inspect/'} />} />
                        <Route path="survey/admin/try/:year" element={<SurveyContainerComponent loadFrom={'/api/survey/try/'} />} />
                        <Route path="survey/respond/:year/:nren" element={<SurveyContainerComponent loadFrom={'/api/survey/load/'} saveTo={'/api/survey/save/'} />} />
                        <Route path="survey/show/:year/:nren" element={<SurveyContainerComponent loadFrom={'/api/survey/load/'} readonly />} />
                        <Route path="*" element={<Landing />} />
                    </Routes>
                </UserProvider>
            </Router>
        </div>
    );
}

export default App;
